@@ -28,6 +28,7 @@
 #include "argsdef.h"
 
 #include "string_opt_arg.h"
+#include "string_opt_arg_free.h"
 #include "int_opt_arg.h"
 #include "long_opt_arg.h"
 #include "long_long_opt_arg.h"
@@ -46,6 +47,7 @@ ConfigParserGenerator::generate_handle_single_option(ostream &stream,
   struct gengetopt_option * opt;
   string_opt_arg_gen_class str_gen;
   str_gen.set_optarg (argstr);
+  string_opt_arg_free_gen_class str_gen_free;
   int_opt_arg_gen_class int_gen;
   int_gen.set_optarg (argstr);
   long_opt_arg_gen_class long_gen;
@@ -55,13 +57,22 @@ ConfigParserGenerator::generate_handle_single_option(ostream &stream,
   float_opt_arg_gen_class float_gen;
   float_gen.set_optarg (argstr);
   flag_opt_arg_gen_class flag_gen;
-
+  int indent_offset = 12;
+  string indent_str (indent + indent_offset, ' ');
+  
   foropt
     {
       ostringstream opt_stream;
-      ostringstream check_stream;
       require_check = true;
 
+      if (opt->type == ARG_NO || opt->type == ARG_FLAG)
+        require_check = false;
+      else
+        {
+          opt_stream << "\n";
+          opt_stream << indent_str;
+        }
+
       switch (opt->type) 
         {
         case ARG_NO:
@@ -75,48 +86,59 @@ ConfigParserGenerator::generate_handle_single_option(ostream &stream,
         case ARG_STRING:
           str_gen.set_opt_var (opt->var_arg);
           SET_GEN_STRUCTURE(str_gen);
-          str_gen.generate_string_opt_arg (opt_stream, indent);
+          if (! opt->multiple && opt->default_given)
+            {
+              str_gen_free.set_structure (ARGS_STRUCT);
+              str_gen_free.set_opt_var (opt->var_arg);
+              str_gen_free.generate_string_opt_arg_free 
+                (opt_stream, indent + indent_offset);
+            }
+          str_gen.generate_string_opt_arg (opt_stream, indent + indent_offset);
           break;
         case ARG_INT:
           int_gen.set_cast ("");
           int_gen.set_opt_var (opt->var_arg);
           SET_GEN_STRUCTURE(int_gen);
-          int_gen.generate_int_opt_arg (opt_stream, indent);
+          int_gen.generate_int_opt_arg (opt_stream, indent + indent_offset);
           break;
         case ARG_SHORT:
           int_gen.set_cast ("(short)");
           int_gen.set_opt_var (opt->var_arg);
           SET_GEN_STRUCTURE(int_gen);
-          int_gen.generate_int_opt_arg (opt_stream, indent);
+          int_gen.generate_int_opt_arg (opt_stream, indent + indent_offset);
           break;
         case ARG_LONG:
           long_gen.set_cast ("");
           long_gen.set_opt_var (opt->var_arg);
           SET_GEN_STRUCTURE(long_gen);
-          long_gen.generate_long_opt_arg (opt_stream, indent);
+          long_gen.generate_long_opt_arg (opt_stream, indent + indent_offset);
           break;
         case ARG_FLOAT:
           float_gen.set_cast ("(float)");
           float_gen.set_opt_var (opt->var_arg);
           SET_GEN_STRUCTURE(float_gen);
-          float_gen.generate_float_opt_arg (opt_stream, indent);
+          float_gen.generate_float_opt_arg 
+            (opt_stream, indent + indent_offset);
           break;
         case ARG_DOUBLE:
           float_gen.set_cast ("");
           float_gen.set_opt_var (opt->var_arg);
           SET_GEN_STRUCTURE(float_gen);
-          float_gen.generate_float_opt_arg (opt_stream, indent);
+          float_gen.generate_float_opt_arg 
+            (opt_stream, indent + indent_offset);
           break;
         case ARG_LONGDOUBLE:
           float_gen.set_cast ("(long double)");
           float_gen.set_opt_var (opt->var_arg);
           SET_GEN_STRUCTURE(float_gen);
-          float_gen.generate_float_opt_arg (opt_stream, indent);
+          float_gen.generate_float_opt_arg 
+            (opt_stream, indent + indent_offset);
           break;
         case ARG_LONGLONG:
           long_long_gen.set_opt_var (opt->var_arg);
           SET_GEN_STRUCTURE(long_long_gen);
-          long_long_gen.generate_long_long_opt_arg (opt_stream, indent);
+          long_long_gen.generate_long_long_opt_arg 
+            (opt_stream, indent + indent_offset);
           break;
         default:
           fprintf (stderr, "gengetopt: bug found in %s:%d\n", __FILE__,
