@@ -24,11 +24,14 @@
 int gengetopt_define_package (char * s) ;
 int gengetopt_define_version (char * s) ;
 int gengetopt_define_purpose (char * s) ;
-int gengetopt_add_group (char * s, int required) ;
+void gengetopt_set_section (const char * s) ;
+int gengetopt_add_group (const char * s, const char *desc, int required) ;
 int gengetopt_add_option (const char * long_opt, char short_opt, 
                           const char * desc, 
                           int type, int flagstat, int required,
                           const char *default_value,
-                          const char * group_value, int multiple);
+                          const char * group_value, 
+                          const char * type_str,
+                          int multiple);
 
 #endif /* _GENGETOPT_H */
