@@ -24,7 +24,11 @@ struct gengetopt_option
   int default_given ; /* if a default is given */
   char * default_string ; /* default value for this option, if string */
   char * group_value; /* group name, if it's part of an option group */
+  char * group_desc; /* group description, if it's part of an option group */
   int multiple; /* whether this option can be given more than once */
+  char *type_str; /* Alternative name for type,
+                     e.g. "URL" or "SECONDS" */
+  char *section; /* the section of this option */
 };
 
 #endif /* _GENGETOPT_GGOS_H */
