@@ -40,7 +40,9 @@ extern "C"
 #include "gm.h"
 #include "my_sstream.h"
 
+#include "groups.h"
 #include "string_opt_arg.h"
+#include "string_opt_arg_free.h"
 #include "int_opt_arg.h"
 #include "long_opt_arg.h"
 #include "long_long_opt_arg.h"
@@ -59,7 +61,9 @@ extern "C"
 #include "config_parser_header.h"
 #include "multiple_opt_list.h"
 #include "multiple_option.h"
+#include "multiple_option_no_short.h"
 #include "multiple_fill_array.h"
+#include "multiple_fill_array_default.h"
 #include "gen_strdup.h"
 
 #include "conf_parser_gen.h"
@@ -88,6 +92,10 @@ static const string to_upper(const string &);
 static void _generate_option_arg(ostream &stream, 
                                  unsigned int indent,
                                  struct gengetopt_option * opt);
+static void 
+generate_help_desc_print(ostream &stream, 
+			 unsigned int desc_column,
+			 const char *descript, const char *defval);
 
 char *
 create_filename (char *name, char *ext)
@@ -213,6 +221,7 @@ CmdlineParserCreator::do_update_arg (struct gengetopt_option *opt,
   const string argstr ("optarg");
   string_opt_arg_gen_class str_gen;
   str_gen.set_optarg (argstr);
+  string_opt_arg_free_gen_class str_gen_free;
   int_opt_arg_gen_class int_gen;
   int_gen.set_optarg (argstr);
   long_opt_arg_gen_class long_gen;
@@ -235,6 +244,7 @@ CmdlineParserCreator::do_update_arg (struct gengetopt_option *opt,
     {
       int_gen.set_structure (ARGS_STRUCT);
       str_gen.set_structure (ARGS_STRUCT);
+      str_gen_free.set_structure (ARGS_STRUCT);
       long_gen.set_structure (ARGS_STRUCT);
       long_long_gen.set_structure (ARGS_STRUCT);
       float_gen.set_structure (ARGS_STRUCT);
@@ -252,6 +262,11 @@ CmdlineParserCreator::do_update_arg (struct gengetopt_option *opt,
       break;
     case ARG_STRING:
       str_gen.set_opt_var (opt->var_arg);
+      if (! opt->multiple && opt->default_given)
+        {
+          str_gen_free.set_opt_var (opt->var_arg);
+          str_gen_free.generate_string_opt_arg_free (stream, indent);
+        }
       str_gen.generate_string_opt_arg (stream, indent);
       generateBreak(stream, indent);
       break;
@@ -509,6 +524,7 @@ CmdlineParserCreator::generate_usage_string()
 {
   struct gengetopt_option * opt;
   ostringstream usage;
+  const char   *type_str;
 
   usage << "Usage: %s ";
 
@@ -516,28 +532,33 @@ CmdlineParserCreator::generate_usage_string()
     {
       foropt
 	if (opt->required) /* required options */
-		switch (opt->type) {
-		case ARG_INT:
-		case ARG_SHORT:
-		case ARG_LONG:
-		case ARG_FLOAT:
-		case ARG_DOUBLE:
-		case ARG_LONGDOUBLE:
-		case ARG_LONGLONG:
-		case ARG_STRING: 
-        	if (opt->short_opt)
-                  {
-                    usage << "-" << opt->short_opt 
-                          << arg_names[opt->type] << "|";
-                  }
-                usage << "--" << opt->long_opt << "=" << 
-                  arg_names[opt->type] << " ";
-                break;
-		default: 
-                  fprintf (stderr, "gengetopt: bug found in %s:%d!!\n",
-                           __FILE__, __LINE__);
-                  abort ();
-		}
+          switch (opt->type) {
+          case ARG_INT:
+          case ARG_SHORT:
+          case ARG_LONG:
+          case ARG_FLOAT:
+          case ARG_DOUBLE:
+          case ARG_LONGDOUBLE:
+          case ARG_LONGLONG:
+          case ARG_STRING: 
+            if (opt->type_str)
+              type_str = opt->type_str;
+            else
+              type_str = arg_names[opt->type];
+
+            if (opt->short_opt)
+              {
+                usage << "-" << opt->short_opt 
+                      << type_str << "|";
+              }
+            usage << "--" << opt->long_opt << "=" << 
+              type_str << " ";
+            break;
+          default: 
+            fprintf (stderr, "gengetopt: bug found in %s:%d!!\n",
+                     __FILE__, __LINE__);
+            abort ();
+          }
       foropt
 	if (!opt->required)
           switch (opt->type) {
@@ -558,13 +579,18 @@ CmdlineParserCreator::generate_usage_string()
           case ARG_LONGDOUBLE:
           case ARG_LONGLONG:
           case ARG_STRING: 
+            if (opt->type_str)
+              type_str = opt->type_str;
+            else
+              type_str = arg_names[opt->type];
+
             if (opt->short_opt)
               {
                 usage << "-" << opt->short_opt 
-                      << arg_names[opt->type] << "|";
+                      << type_str << "|";
               }
             usage << "--" << opt->long_opt << "=" << 
-              arg_names[opt->type] << " ";
+              type_str << " ";
             break;
           default: fprintf (stderr, "gengetopt: bug found in %s:%d!!\n",
                             __FILE__, __LINE__);
@@ -584,94 +610,209 @@ CmdlineParserCreator::generate_usage_string()
   return usage.str ();
 }
 
+inline static bool
+char_is_newline(const char *buf)
+{
+  return (*buf == '\\' && (buf+1) != 0 && *(buf+1) == 'n');
+}
+
+static void 
+generate_help_desc_print(ostream &stream, 
+			 unsigned int desc_column,
+			 const char *descript, const char *defval)
+{
+  int  next_space = desc_column;
+  char next_word[128];
+  int  nw_ind = 0;
+
+  string desc_indent (desc_column + 2, ' ');
+  string desc = descript;
+
+  if (defval)
+    {
+      desc += "  (default=";
+      desc += defval;
+      desc += ")";
+    }
+
+  const char * out_buf = desc.c_str();
+  bool found_new_line;
+
+  while (*out_buf)
+    {
+      found_new_line = false;
+      // search next whitespace
+      while ((*out_buf) && (*out_buf != ' ') && 
+	     (*out_buf != '=') && ! char_is_newline(out_buf))
+	{
+	  next_word[nw_ind++] = *out_buf++;
+	  next_space++;
+	}
+      if (*out_buf)
+	{
+	  if (char_is_newline (out_buf))
+            {
+              found_new_line = true;
+              next_space = 79;
+              out_buf++; // \n is two chars
+            }
+	  else
+            next_word[nw_ind++] = *out_buf;
+
+	  out_buf++;
+	}
+
+      next_word[nw_ind] = 0;
+      next_space++;
+
+      // wrap line if it's too long
+      if (next_space > 79)
+	{
+          if (found_new_line)
+            stream << next_word;
+	  stream << "\\n" << desc_indent;
+	  next_space = desc_column + 2;
+          if (! found_new_line)
+            next_space += strlen (next_word);
+	}
+
+      // print word
+      if (! found_new_line)
+        stream << next_word;
+      nw_ind = 0;
+    }
+}
+
 void 
 CmdlineParserCreator::generate_help_option_print(ostream &stream, 
                                                  unsigned int indent)
 {
-  long max_long, max_short, w;
+  long desc_col;
   struct gengetopt_option * opt;
   string indent_str (indent, ' ');
 
+  int		type_len;
+  const char   *type_str;
+
   /* calculate columns */
-  max_long = max_short = 0;
+  desc_col = 0;
   foropt {
-    w = 3 + strlen (opt->long_opt);
-    if (opt->type == ARG_FLAG || opt->type == ARG_NO)
+    int width = 2 + 4 + 2;  // ws + "-a, " + ws
+
+    width += strlen (opt->long_opt) + 2;  // "--"
+
+    if ((opt->type != ARG_FLAG) &&
+	(opt->type != ARG_NO))
       {
-        if (w > max_long) max_long = w;
-        if (2 > max_short) max_short = 2;
-      }
-    else
-      {
-        w += strlen (arg_names[opt->type]);
-        if (w > max_long) max_long = w;
-        w = (3 + strlen (arg_names[opt->type]));
-        if (w > max_short) max_short = w;
+	if (opt->type_str)
+	  type_str = opt->type_str;
+	else
+	  type_str = arg_names[opt->type];
+	type_len = strlen(type_str);
+	
+	width += type_len + 1;	// "="
       }
+
+    if (width > desc_col)
+      desc_col = width;
   }
+
   /* print justified options */
   char *prev_group = 0;
   print_group_gen_class print_group;
+  char *curr_section = 0;
+  bool first_option = true;
+
+  stream << indent_str;
+  stream << "printf(\"\\n\");\n";
 
   foropt
     {
-      if (opt->group_value &&
+      if (opt->group_value && 
           (! prev_group || strcmp (opt->group_value, prev_group) != 0))
         {
           stream << indent_str;
           print_group.set_group_name (opt->group_value);
+          print_group.set_group_desc (opt->group_desc);
           print_group.generate_print_group (stream, indent);
           stream << endl;
 
           prev_group = opt->group_value;
         }
 
+      if (opt->section && 
+          (!curr_section || strcmp (curr_section, opt->section)))
+        {
+          curr_section = opt->section;
+          stream << indent_str;
+          stream << "printf(\"";
+          if (! first_option)
+            stream << "\\n";
+          stream << opt->section << ":";
+          stream << "\\n\");\n";
+        }
+
+      first_option = false;
+
       stream << indent_str;
       stream << "printf(\"";
       if (opt->type == ARG_FLAG || opt->type == ARG_NO)
         {
+	  const char * def_val = NULL;
+	  
           if (opt->short_opt) 
-            stream << "   -" << opt->short_opt;
+	    stream << "  -" << opt->short_opt << ", ";
           else
-            stream << "     ";
-          for (w = 2; w < max_short; w++) 
-            stream << " ";
-          stream << "  --" << opt->long_opt;
-          for (w = 2+strlen(opt->long_opt); w < max_long; w++)
-            stream << " ";
-          stream << "  " << opt->desc;
-          if (opt->type == ARG_FLAG)
-            {
-              if (opt->flagstat)
-                stream << " (default=on)";
-              else
-                stream << " (default=off)";
-            }
+	    stream << "      ";
+
+          stream << "--" << opt->long_opt;
+
+	  string desc_indent (desc_col - 8 - strlen(opt->long_opt), ' ');
+	  stream << desc_indent;
+
+	  if (opt->type == ARG_FLAG)
+	    def_val = opt->flagstat ? "on" : "off";
+	  
+	  generate_help_desc_print(stream, desc_col, opt->desc, def_val);
           stream << "\\n\");\n";
         }
       else
         {
+	  int col = 0;
+	  string def_str = "`";
+	  const char * def_val = NULL;
+
+	  if (opt->type_str)
+	    type_str = opt->type_str;
+	  else
+	    type_str = arg_names[opt->type];
+	  type_len = strlen(type_str);
+
           if (opt->short_opt)
-            stream << "   -" << opt->short_opt << arg_names[opt->type];
+	    {
+	      stream << "  -" << opt->short_opt << ", ";
+	      col += 6;
+	    }
           else
             {
-              int type_len = strlen(arg_names[opt->type]);
-              
               stream << "      ";
-              for (w = 1; w < type_len; w++) 
-                stream << " ";
-            }
-          for (w = 2+strlen(arg_names[opt->type]); w < max_short; w++)
-            stream << " ";
-          stream << "  --" << opt->long_opt << "=" << arg_names[opt->type];
-          for (w = 3+strlen(opt->long_opt)+
-                 strlen(arg_names[opt->type]); w < max_long; w++)
-            stream << " ";
-          stream << "  " << opt->desc;
-          if (opt->default_given)
-            {
-              stream << " (default='" << opt->default_string << "')";
+	      col += 6;
             }
+
+          stream << "--" << opt->long_opt << "=" << type_str;
+	  col += 3 + strlen(opt->long_opt) + type_len;
+	  
+	  string desc_indent (desc_col - col, ' ');
+	  stream << desc_indent;
+
+	  if (opt->default_string)
+	    {
+	      def_str += opt->default_string;
+	      def_str += "'";
+	      def_val = def_str.c_str();
+	    }
+
+	  generate_help_desc_print(stream, desc_col, opt->desc, def_val);
           stream << "\\n\");\n";
         }
     }
@@ -762,6 +903,21 @@ CmdlineParserCreator::generate_multiple_fill_array(ostream &stream, unsigned int
 	  filler.set_type (arg_types[opt->type]);
 	  filler.set_option_var_name (opt->var_arg);
 	  filler.generate_multiple_fill_array (stream, indent);
+
+          if (opt->default_given)
+            {
+              multiple_fill_array_default_gen_class def_filler;
+              string def_value = opt->default_string;
+
+              def_filler.set_type (arg_types[opt->type]);
+              def_filler.set_option_var_name (opt->var_arg);
+              if (opt->type == ARG_STRING)
+                def_value = "gengetopt_strdup(\"" +
+                  def_value + "\")";
+              def_filler.set_default_value (def_value);
+              def_filler.generate_multiple_fill_array_default (stream, indent);
+            }
+
           stream << endl;
 	}
     }
@@ -776,7 +932,17 @@ CmdlineParserCreator::generate_clear_arg(ostream &stream, unsigned int indent)
   /* now we initialize "given" fields */
   foropt
     {
-      if (opt->type == ARG_STRING)
+      if (opt->multiple)
+        {
+          // since multiple options are stored in arrays, we
+          // always initialize it to NULL
+
+          stream << indent_str;
+
+          stream << ARGS_STRUCT << "->" << opt->var_arg 
+                 << "_arg = NULL; \\" << endl;
+        }
+      else if (opt->type == ARG_STRING)
         {
           stream << indent_str;
 
@@ -931,6 +1097,7 @@ CmdlineParserCreator::generate_handle_no_short_option(ostream &stream,
   string indent_str (indent, ' ');
   opt_gen.set_gen_exit (no_handle_error ? "return" : "exit");
   opt_gen.set_package_var_name (c_source_gen_class::package_var_name);
+  multiple_option_no_short_gen_class multip_opt_gen;
   bool first = true;
 
   foropt
@@ -939,25 +1106,55 @@ CmdlineParserCreator::generate_handle_no_short_option(ostream &stream,
         stream << indent_str;
         ostringstream str_stream;
 
-        opt_gen.set_option_comment (opt->desc);
-        opt_gen.set_long_option (opt->long_opt);
-        opt_gen.set_option_var_name (opt->var_arg);
-
-        do_update_arg (opt, str_stream, indent + 2);
-        opt_gen.set_update_arg (str_stream.str ());
-
-        generic_option_group_gen_class group_gen;
-        if (opt->group_value)
+        if (opt->multiple)
           {
-            str_stream.str ("");
-            str_stream << " ";
-            group_gen.set_group_var_name 
-              (canonize_name (opt->group_value));
-            group_gen.generate_generic_option_group (str_stream, indent);
-            opt_gen.set_update_group_count (str_stream.str ());
-          }
+            multip_opt_gen.set_option_comment (opt->desc);
+            multip_opt_gen.set_long_option (opt->long_opt);
+            multip_opt_gen.set_option_var_name (opt->var_arg);
 
-        opt_gen.generate_no_short_option (stream, indent);
+            do_update_arg (opt, str_stream, indent + 2);
+            multip_opt_gen.set_update_arg (str_stream.str ());
+
+            generic_option_group_gen_class group_gen;
+            if (opt->group_value)
+              {
+                str_stream.str ("");
+                str_stream << " ";
+                group_gen.set_group_var_name 
+                  (canonize_name (opt->group_value));
+                group_gen.generate_generic_option_group (str_stream, indent);
+                multip_opt_gen.set_update_group_count (str_stream.str ());
+              }
+            else
+              multip_opt_gen.set_update_group_count ("");
+
+            multip_opt_gen.generate_multiple_option_no_short
+              (stream, indent);
+          }
+        else
+          {
+            opt_gen.set_option_comment (opt->desc);
+            opt_gen.set_long_option (opt->long_opt);
+            opt_gen.set_option_var_name (opt->var_arg);
+
+            do_update_arg (opt, str_stream, indent + 2);
+            opt_gen.set_update_arg (str_stream.str ());
+
+            generic_option_group_gen_class group_gen;
+            if (opt->group_value)
+              {
+                str_stream.str ("");
+                str_stream << " ";
+                group_gen.set_group_var_name 
+                  (canonize_name (opt->group_value));
+                group_gen.generate_generic_option_group (str_stream, indent);
+                opt_gen.set_update_group_count (str_stream.str ());
+              }
+            else
+              opt_gen.set_update_group_count ("");
+
+            opt_gen.generate_no_short_option (stream, indent);
+          }
 
         stream << endl;
 
@@ -965,6 +1162,7 @@ CmdlineParserCreator::generate_handle_no_short_option(ostream &stream,
           {
             first = false;
             opt_gen.set_gen_else ("else ");
+            multip_opt_gen.set_gen_else ("else ");
           }
       }
 
@@ -1050,7 +1248,7 @@ CmdlineParserCreator::generate_handle_group(ostream &stream,
       stream << indent_str;
       opt_gen.set_group_name (idx->first);
       opt_gen.set_group_var_name (canonize_name (idx->first));
-      if (idx->second)
+      if (idx->second.required)
         {
           opt_gen.set_Comparison_rule(GROUP_REQUIRED_COMPARISON);
           opt_gen.set_number_required(GROUP_REQUIRED_MESSAGE);
