@@ -21,16 +21,11 @@
 #ifndef _GM_H
 #define _GM_H
 
-#include "my_string.h"
-#include "my_map.h"
-
 #include "header.h"
 #include "c_source.h"
 
 #define TAB_LEN 2
 
-typedef map<string,bool> groups_collection_t;
-
 struct gengetopt_option;
 
 class CmdlineParserCreator : public header_gen_class, public c_source_gen_class
