@@ -17,7 +17,7 @@ extern char linebuf[];
 extern char *yytext;
 
 void
-yyerror (char *s)
+yyerror (const char *s)
 {
   const char *source =
     (gengetopt_input_filename ? gengetopt_input_filename : "gengetopt");
@@ -32,7 +32,7 @@ yyerror (char *s)
 // but there's a "semantic" error.  Indeed the line number has to be
 // decremented.
 void
-yyerror2 (char *s)
+yyerror2 (const char *s)
 {
   const char *source =
     (gengetopt_input_filename ? gengetopt_input_filename : "gengetopt");
