@@ -16,7 +16,7 @@ main (int argc, char **argv)
     exit(1) ;
 
   if (test_conf_parser_cmd_parser_configfile
-      (args_info.conf_file_arg, &args_info, 1) != 0) // override cmd options
+      (args_info.conf_file_arg, &args_info, 1) != 0) /* override cmd options */
     exit(1);
 
   printf ("value of required: %s\n", args_info.required_arg);
